@@ -1,29 +1,21 @@
-# -*- coding: utf-8 -*-
-"""
-    tests.http
-    ~~~~~~~~~~
-
-    HTTP parsing utilities.
-
-    :copyright: 2007 Pallets
-    :license: BSD-3-Clause
-"""
+import base64
+from datetime import date
 from datetime import datetime
+from datetime import timedelta
+from datetime import timezone
 
 import pytest
 
-from . import strict_eq
 from werkzeug import datastructures
 from werkzeug import http
-from werkzeug._compat import itervalues
-from werkzeug._compat import wsgi_encoding_dance
+from werkzeug._internal import _wsgi_encoding_dance
 from werkzeug.test import create_environ
 
 
-class TestHTTPUtility(object):
+class TestHTTPUtility:
     def test_accept(self):
         a = http.parse_accept_header("en-us,ru;q=0.5")
-        assert list(itervalues(a)) == ["en-us", "ru"]
+        assert list(a.values()) == ["en-us", "ru"]
         assert a.best == "en-us"
         assert a.find("ru") == 1
         pytest.raises(ValueError, a.index, "de")
@@ -123,6 +115,13 @@ class TestHTTPUtility(object):
         assert c.private == "*"
         del c.private
         assert c.private is None
+        # max_age is an int, other types are converted
+        c.max_age = 3.1
+        assert c.max_age == 3
+        del c.max_age
+        c.s_maxage = 3.1
+        assert c.s_maxage == 3
+        del c.s_maxage
         assert c.to_header() == "no-cache"
 
     def test_csp_header(self):
@@ -136,20 +135,20 @@ class TestHTTPUtility(object):
     def test_authorization_header(self):
         a = http.parse_authorization_header("Basic QWxhZGRpbjpvcGVuIHNlc2FtZQ==")
         assert a.type == "basic"
-        assert a.username == u"Aladdin"
-        assert a.password == u"open sesame"
+        assert a.username == "Aladdin"
+        assert a.password == "open sesame"
 
         a = http.parse_authorization_header(
             "Basic 0YDRg9GB0YHQutC40IE60JHRg9C60LLRiw=="
         )
         assert a.type == "basic"
-        assert a.username == u"русскиЁ"
-        assert a.password == u"Буквы"
+        assert a.username == "русскиЁ"
+        assert a.password == "Буквы"
 
         a = http.parse_authorization_header("Basic 5pmu6YCa6K+dOuS4reaWhw==")
         assert a.type == "basic"
-        assert a.username == u"普通话"
-        assert a.password == u"中文"
+        assert a.username == "普通话"
+        assert a.password == "中文"
 
         a = http.parse_authorization_header(
             '''Digest username="Mufasa",
@@ -193,6 +192,11 @@ class TestHTTPUtility(object):
         assert http.parse_authorization_header(None) is None
         assert http.parse_authorization_header("foo") is None
 
+    def test_bad_authorization_header_encoding(self):
+        """If the base64 encoded bytes can't be decoded as UTF-8"""
+        content = base64.b64encode(b"\xffser:pass").decode()
+        assert http.parse_authorization_header(f"Basic {content}") is None
+
     def test_www_authenticate_header(self):
         wa = http.parse_www_authenticate_header('Basic realm="WallyWorld"')
         assert wa.type == "basic"
@@ -245,27 +249,6 @@ class TestHTTPUtility(object):
         assert bool(etags)
         assert etags.contains_raw('W/"foo"')
 
-    def test_parse_date(self):
-        assert http.parse_date("Sun, 06 Nov 1994 08:49:37 GMT    ") == datetime(
-            1994, 11, 6, 8, 49, 37
-        )
-        assert http.parse_date("Sunday, 06-Nov-94 08:49:37 GMT") == datetime(
-            1994, 11, 6, 8, 49, 37
-        )
-        assert http.parse_date(" Sun Nov  6 08:49:37 1994") == datetime(
-            1994, 11, 6, 8, 49, 37
-        )
-        assert http.parse_date("foo") is None
-
-    def test_parse_date_overflows(self):
-        assert http.parse_date(" Sun 02 Feb 1343 08:49:37 GMT") == datetime(
-            1343, 2, 2, 8, 49, 37
-        )
-        assert http.parse_date("Thu, 01 Jan 1970 00:00:00 GMT") == datetime(
-            1970, 1, 1, 0, 0
-        )
-        assert http.parse_date("Thu, 33 Jan 1970 00:00:00 GMT") is None
-
     def test_remove_entity_headers(self):
         now = http.http_date()
         headers1 = [
@@ -279,7 +262,7 @@ class TestHTTPUtility(object):
         assert headers1 == [("Date", now)]
 
         http.remove_entity_headers(headers2)
-        assert headers2 == datastructures.Headers([(u"Date", now)])
+        assert headers2 == datastructures.Headers([("Date", now)])
 
     def test_remove_hop_by_hop_headers(self):
         headers1 = [("Connection", "closed"), ("Foo", "bar"), ("Keep-Alive", "wtf")]
@@ -314,29 +297,34 @@ class TestHTTPUtility(object):
         )
         # Issue #404
         assert http.parse_options_header(
-            'multipart/form-data; name="foo bar"; ' 'filename="bar foo"'
+            'multipart/form-data; name="foo bar"; filename="bar foo"'
         ) == ("multipart/form-data", {"name": "foo bar", "filename": "bar foo"})
         # Examples from RFC
         assert http.parse_options_header("audio/*; q=0.2, audio/basic") == (
             "audio/*",
             {"q": "0.2"},
         )
-        assert http.parse_options_header(
-            "audio/*; q=0.2, audio/basic", multiple=True
-        ) == ("audio/*", {"q": "0.2"}, "audio/basic", {})
-        assert http.parse_options_header(
-            "text/plain; q=0.5, text/html\n        text/x-dvi; q=0.8, text/x-c",
-            multiple=True,
-        ) == (
-            "text/plain",
-            {"q": "0.5"},
-            "text/html",
-            {},
-            "text/x-dvi",
-            {"q": "0.8"},
-            "text/x-c",
-            {},
-        )
+
+        with pytest.warns(DeprecationWarning):
+            assert http.parse_options_header(
+                "audio/*; q=0.2, audio/basic", multiple=True
+            ) == ("audio/*", {"q": "0.2"}, "audio/basic", {})
+
+        with pytest.warns(DeprecationWarning):
+            assert http.parse_options_header(
+                "text/plain; q=0.5, text/html\n        text/x-dvi; q=0.8, text/x-c",
+                multiple=True,
+            ) == (
+                "text/plain",
+                {"q": "0.5"},
+                "text/html",
+                {},
+                "text/x-dvi",
+                {"q": "0.8"},
+                "text/x-c",
+                {},
+            )
+
         assert http.parse_options_header(
             "text/plain; q=0.5, text/html\n        text/x-dvi; q=0.8, text/x-c"
         ) == ("text/plain", {"q": "0.5"})
@@ -344,13 +332,13 @@ class TestHTTPUtility(object):
         assert http.parse_options_header(
             "form-data; name=\"a_file\"; filename*=UTF-8''"
             '"%c2%a3%20and%20%e2%82%ac%20rates"'
-        ) == ("form-data", {"name": "a_file", "filename": u"\xa3 and \u20ac rates"})
+        ) == ("form-data", {"name": "a_file", "filename": "\xa3 and \u20ac rates"})
         assert http.parse_options_header(
             "form-data; name*=UTF-8''\"%C5%AAn%C4%ADc%C5%8Dde%CC%BD\"; "
             'filename="some_file.txt"'
         ) == (
             "form-data",
-            {"name": u"\u016an\u012dc\u014dde\u033d", "filename": "some_file.txt"},
+            {"name": "\u016an\u012dc\u014dde\u033d", "filename": "some_file.txt"},
         )
 
     def test_parse_options_header_value_with_quotes(self):
@@ -359,7 +347,7 @@ class TestHTTPUtility(object):
         ) == ("form-data", {"name": "file", "filename": "t'es't.txt"})
         assert http.parse_options_header(
             "form-data; name=\"file\"; filename*=UTF-8''\"'🐍'.txt\""
-        ) == ("form-data", {"name": "file", "filename": u"'🐍'.txt"})
+        ) == ("form-data", {"name": "file", "filename": "'🐍'.txt"})
 
     def test_parse_options_header_broken_values(self):
         # Issue #995
@@ -436,32 +424,26 @@ class TestHTTPUtility(object):
             env, last_modified=datetime(2008, 1, 1, 13, 30), ignore_if_range=True
         )
 
-    def test_date_formatting(self):
-        assert http.cookie_date(0) == "Thu, 01-Jan-1970 00:00:00 GMT"
-        assert http.cookie_date(datetime(1970, 1, 1)) == "Thu, 01-Jan-1970 00:00:00 GMT"
-        assert http.http_date(0) == "Thu, 01 Jan 1970 00:00:00 GMT"
-        assert http.http_date(datetime(1970, 1, 1)) == "Thu, 01 Jan 1970 00:00:00 GMT"
-
     def test_parse_cookie(self):
         cookies = http.parse_cookie(
             "dismiss-top=6; CP=null*; PHPSESSID=0a539d42abc001cdc762809248d4beed;"
-            ' a=42; b="\\";"; ; fo234{=bar;blub=Blah;'
+            'a=42; b="\\";"; ; fo234{=bar;blub=Blah; "__Secure-c"=d'
         )
         assert cookies.to_dict() == {
-            "CP": u"null*",
-            "PHPSESSID": u"0a539d42abc001cdc762809248d4beed",
-            "a": u"42",
-            "dismiss-top": u"6",
-            "b": u'";',
-            "fo234{": u"bar",
-            "blub": u"Blah",
+            "CP": "null*",
+            "PHPSESSID": "0a539d42abc001cdc762809248d4beed",
+            "a": "42",
+            "dismiss-top": "6",
+            "b": '";',
+            "fo234{": "bar",
+            "blub": "Blah",
+            '"__Secure-c"': "d",
         }
 
     def test_dump_cookie(self):
         rv = http.dump_cookie(
             "foo", "bar baz blub", 360, httponly=True, sync_expires=False
         )
-        assert type(rv) is str
         assert set(rv.split("; ")) == {
             "HttpOnly",
             "Max-Age=360",
@@ -476,66 +458,64 @@ class TestHTTPUtility(object):
             "first=IamTheFirst ; a=1; oops ; a=2 ;second = andMeTwo;"
         )
         expect = {
-            "first": [u"IamTheFirst"],
-            "a": [u"1", u"2"],
-            "oops": [u""],
-            "second": [u"andMeTwo"],
+            "first": ["IamTheFirst"],
+            "a": ["1", "2"],
+            "oops": [""],
+            "second": ["andMeTwo"],
         }
         assert cookies.to_dict(flat=False) == expect
-        assert cookies["a"] == u"1"
-        assert cookies.getlist("a") == [u"1", u"2"]
+        assert cookies["a"] == "1"
+        assert cookies.getlist("a") == ["1", "2"]
 
     def test_empty_keys_are_ignored(self):
         cookies = http.parse_cookie("spam=ham; duck=mallard; ; ")
-        expect = {"spam": u"ham", "duck": u"mallard"}
+        expect = {"spam": "ham", "duck": "mallard"}
         assert cookies.to_dict() == expect
 
     def test_cookie_quoting(self):
         val = http.dump_cookie("foo", "?foo")
         assert val == 'foo="?foo"; Path=/'
-        assert http.parse_cookie(val).to_dict() == {"foo": u"?foo", "Path": u"/"}
-        assert http.parse_cookie(r'foo="foo\054bar"').to_dict(), {"foo": u"foo,bar"}
+        assert http.parse_cookie(val).to_dict() == {"foo": "?foo", "Path": "/"}
+        assert http.parse_cookie(r'foo="foo\054bar"').to_dict(), {"foo": "foo,bar"}
 
     def test_parse_set_cookie_directive(self):
         val = 'foo="?foo"; version="0.1";'
-        assert http.parse_cookie(val).to_dict() == {"foo": u"?foo", "version": u"0.1"}
+        assert http.parse_cookie(val).to_dict() == {"foo": "?foo", "version": "0.1"}
 
     def test_cookie_domain_resolving(self):
-        val = http.dump_cookie("foo", "bar", domain=u"\N{SNOWMAN}.com")
-        strict_eq(val, "foo=bar; Domain=xn--n3h.com; Path=/")
+        val = http.dump_cookie("foo", "bar", domain="\N{SNOWMAN}.com")
+        assert val == "foo=bar; Domain=xn--n3h.com; Path=/"
 
     def test_cookie_unicode_dumping(self):
-        val = http.dump_cookie("foo", u"\N{SNOWMAN}")
+        val = http.dump_cookie("foo", "\N{SNOWMAN}")
         h = datastructures.Headers()
         h.add("Set-Cookie", val)
         assert h["Set-Cookie"] == 'foo="\\342\\230\\203"; Path=/'
 
         cookies = http.parse_cookie(h["Set-Cookie"])
-        assert cookies["foo"] == u"\N{SNOWMAN}"
+        assert cookies["foo"] == "\N{SNOWMAN}"
 
     def test_cookie_unicode_keys(self):
         # Yes, this is technically against the spec but happens
-        val = http.dump_cookie(u"fö", u"fö")
-        assert val == wsgi_encoding_dance(u'fö="f\\303\\266"; Path=/', "utf-8")
+        val = http.dump_cookie("fö", "fö")
+        assert val == _wsgi_encoding_dance('fö="f\\303\\266"; Path=/', "utf-8")
         cookies = http.parse_cookie(val)
-        assert cookies[u"fö"] == u"fö"
+        assert cookies["fö"] == "fö"
 
     def test_cookie_unicode_parsing(self):
-        # This is actually a correct test.  This is what is being submitted
-        # by firefox if you set an unicode cookie and we get the cookie sent
-        # in on Python 3 under PEP 3333.
-        cookies = http.parse_cookie(u"fÃ¶=fÃ¶")
-        assert cookies[u"fö"] == u"fö"
+        # This is submitted by Firefox if you set a Unicode cookie.
+        cookies = http.parse_cookie("fÃ¶=fÃ¶")
+        assert cookies["fö"] == "fö"
 
     def test_cookie_domain_encoding(self):
-        val = http.dump_cookie("foo", "bar", domain=u"\N{SNOWMAN}.com")
-        strict_eq(val, "foo=bar; Domain=xn--n3h.com; Path=/")
+        val = http.dump_cookie("foo", "bar", domain="\N{SNOWMAN}.com")
+        assert val == "foo=bar; Domain=xn--n3h.com; Path=/"
 
-        val = http.dump_cookie("foo", "bar", domain=u".\N{SNOWMAN}.com")
-        strict_eq(val, "foo=bar; Domain=.xn--n3h.com; Path=/")
+        val = http.dump_cookie("foo", "bar", domain=".\N{SNOWMAN}.com")
+        assert val == "foo=bar; Domain=.xn--n3h.com; Path=/"
 
-        val = http.dump_cookie("foo", "bar", domain=u".foo.com")
-        strict_eq(val, "foo=bar; Domain=.foo.com; Path=/")
+        val = http.dump_cookie("foo", "bar", domain=".foo.com")
+        assert val == "foo=bar; Domain=.foo.com; Path=/"
 
     def test_cookie_maxsize(self, recwarn):
         val = http.dump_cookie("foo", "bar" * 1360 + "b")
@@ -570,7 +550,7 @@ class TestHTTPUtility(object):
             http.dump_cookie("foo", "bar", samesite="invalid")
 
 
-class TestRange(object):
+class TestRange:
     def test_if_range_parsing(self):
         rv = http.parse_if_range_header('"Test"')
         assert rv.etag == "Test"
@@ -591,7 +571,7 @@ class TestRange(object):
 
         rv = http.parse_if_range_header("Thu, 01 Jan 1970 00:00:00 GMT")
         assert rv.etag is None
-        assert rv.date == datetime(1970, 1, 1)
+        assert rv.date == datetime(1970, 1, 1, tzinfo=timezone.utc)
         assert rv.to_header() == "Thu, 01 Jan 1970 00:00:00 GMT"
 
         for x in "", None:
@@ -669,7 +649,7 @@ class TestRange(object):
         assert rv.units == "bytes"
 
 
-class TestRegression(object):
+class TestRegression:
     def test_best_match_works(self):
         # was a bug in 0.6
         rv = http.parse_accept_header(
@@ -679,3 +659,75 @@ class TestRegression(object):
             datastructures.MIMEAccept,
         ).best_match(["foo/bar"])
         assert rv == "foo/bar"
+
+
+@pytest.mark.parametrize(
+    "value",
+    [
+        "Basic V2Vya3pldWc6V2VrcnpldWc=",
+        'Digest username=Mufasa, realm="testrealm@host.invalid",'
+        ' nonce=dcd98b7102dd2f0e8b11d0f600bfb0c093, uri="/dir/index.html", qop=auth,'
+        " nc=00000001, cnonce=0a4f113b, response=6629fae49393a05397450978507c4ef1,"
+        " opaque=5ccc069c403ebaf9f0171e9517f40e41",
+    ],
+)
+def test_authorization_to_header(value: str) -> None:
+    parsed = http.parse_authorization_header(value)
+    assert parsed is not None
+    assert parsed.to_header() == value
+
+
+@pytest.mark.parametrize(
+    ("value", "expect"),
+    [
+        (
+            "Sun, 06 Nov 1994 08:49:37 GMT    ",
+            datetime(1994, 11, 6, 8, 49, 37, tzinfo=timezone.utc),
+        ),
+        (
+            "Sunday, 06-Nov-94 08:49:37 GMT",
+            datetime(1994, 11, 6, 8, 49, 37, tzinfo=timezone.utc),
+        ),
+        (
+            " Sun Nov  6 08:49:37 1994",
+            datetime(1994, 11, 6, 8, 49, 37, tzinfo=timezone.utc),
+        ),
+        ("foo", None),
+        (
+            " Sun 02 Feb 1343 08:49:37 GMT",
+            datetime(1343, 2, 2, 8, 49, 37, tzinfo=timezone.utc),
+        ),
+        (
+            "Thu, 01 Jan 1970 00:00:00 GMT",
+            datetime(1970, 1, 1, tzinfo=timezone.utc),
+        ),
+        ("Thu, 33 Jan 1970 00:00:00 GMT", None),
+    ],
+)
+def test_parse_date(value, expect):
+    assert http.parse_date(value) == expect
+
+
+@pytest.mark.parametrize(
+    ("value", "expect"),
+    [
+        (
+            datetime(1994, 11, 6, 8, 49, 37, tzinfo=timezone.utc),
+            "Sun, 06 Nov 1994 08:49:37 GMT",
+        ),
+        (
+            datetime(1994, 11, 6, 8, 49, 37, tzinfo=timezone(timedelta(hours=-8))),
+            "Sun, 06 Nov 1994 16:49:37 GMT",
+        ),
+        (datetime(1994, 11, 6, 8, 49, 37), "Sun, 06 Nov 1994 08:49:37 GMT"),
+        (0, "Thu, 01 Jan 1970 00:00:00 GMT"),
+        (datetime(1970, 1, 1), "Thu, 01 Jan 1970 00:00:00 GMT"),
+        (datetime(1, 1, 1), "Mon, 01 Jan 0001 00:00:00 GMT"),
+        (datetime(999, 1, 1), "Tue, 01 Jan 0999 00:00:00 GMT"),
+        (datetime(1000, 1, 1), "Wed, 01 Jan 1000 00:00:00 GMT"),
+        (datetime(2020, 1, 1), "Wed, 01 Jan 2020 00:00:00 GMT"),
+        (date(2020, 1, 1), "Wed, 01 Jan 2020 00:00:00 GMT"),
+    ],
+)
+def test_http_date(value, expect):
+    assert http.http_date(value) == expect
